/* eslint-disable import/no-extraneous-dependencies */
import express from "express";
import { config } from "dotenv";
import helmet from "helmet";
import rateLimit from "express-rate-limit";
import session from "express-session";
import cookieParser from "cookie-parser";
import bodyParser from "body-parser";
import path from "node:path";
import morgan from "morgan";
import flash from "express-flash";
import passport from "passport";
import { Strategy as SpotifyStrategy } from "passport-spotify";

import router from "./router.js";

config(); // Load environment variables from .env file

// file deepcode ignore UseCsurfForExpress: We are using tiny-csrf.
const app = express();
app.set(`trust proxy`, 1); // trust first proxy
const port = process.env.PORT || 3000;
const cookieSecret = process.env.COOKIE_SECRET;
const sessionSecret = process.env.SESSION_SECRET;

const baseUrl = process.env.BASE_URL;
let secureFlag = false;
if (baseUrl.startsWith(`https://`))
    secureFlag = true;

// Hide ?code=[...] from logs
morgan.token(`url`, (request) =>
{
    if (request.query && request.query.code)
        return request.originalUrl.replace(`code=${ request.query.code }`, `code=[...]`);
    return request.originalUrl || request.url;
});
app.use(morgan(secureFlag ? `combined` : `dev`));

app.use(bodyParser.json());
app.use(express.urlencoded({ extended: true }));
app.use(cookieParser(cookieSecret));
app.use(session({
    secret: sessionSecret,
    resave: false,
    saveUninitialized: false,
    // file deepcode ignore WebCookieSecureDisabledExplicitly: We disable secure if baseUrl does not start with https.
    cookie: {
        httpOnly: true,
        secure: secureFlag,
        maxAge: 1000 * 60 * 60 * 24 * 7, // 7d
    },
}));
app.use(flash());
app.use(passport.initialize());
app.use(passport.session());

app.use(express.static(`./src/public`)); // Serve static files from public on root (projectRoot/src/public/js -> example.com/js)
// hotload jquery, bootstrap
const projectRoot = path.resolve(`.`);
app.use(`/css`, express.static(path.join(projectRoot, `node_modules/bootstrap/dist/css`)));
app.use(`/js`, express.static(path.join(projectRoot, `node_modules/bootstrap/dist/js`)));
app.use(`/js`, express.static(path.join(projectRoot, `node_modules/jquery/dist`)));
app.use(`/css`, express.static(path.join(projectRoot, `node_modules/@fortawesome/fontawesome-free/css`)));
app.use(`/webfonts`, express.static(path.join(projectRoot, `node_modules/@fortawesome/fontawesome-free/webfonts`)));
app.use(helmet({
    contentSecurityPolicy: {
        directives: {
            ...helmet.contentSecurityPolicy.getDefaultDirectives(),
            "img-src": [`'self'`, `data:`],
            "script-src": [`'self'`, `https://*.chse.dev`],
            "default-src": [`'self'`, `https://*.chse.dev`],
        },
    },
})); // Apply helmet to all requests

const limiter = rateLimit({
    windowMs: 15 * 60 * 1000, // 15 minutes
    max: 100, // Limit each IP to 100 requests per `window` (here, per 15 minutes)
    standardHeaders: true, // Return rate limit info in the `RateLimit-*` headers
});
app.use(limiter); // Apply rate limiter to all requests

/**
 * @name initializePassport
 * @description Initialize passport
 * @param {*} passport Passport.js Object
 */
function initializePassport(passport)
{
    if (process.env.SPOTIFY_CLIENT_ID && process.env.SPOTIFY_CLIENT_SECRET)
    {
        /* eslint-disable camelcase */
        passport.use(new SpotifyStrategy(
            {
                clientID: process.env.SPOTIFY_CLIENT_ID,
                clientSecret: process.env.SPOTIFY_CLIENT_SECRET,
                callbackURL: `${ baseUrl }/login/spotify/callback`,
                scope: [`user-top-read`],
            },
            (accessToken, refreshToken, expires_in, profile, done) => done(undefined, accessToken)
        ));
    }

    /* eslint-enable camelcase */

    passport.serializeUser((user, done) =>
    {
        done(undefined, user);
    });

    passport.deserializeUser((user, done) =>
    {
        done(undefined, user);
    });
}

initializePassport(
    passport,
);

app.set(`views`, `./src/views`);
app.set(`view engine`, `ejs`);

app.use(router); // Apply router from src/router.js

app.listen(port, () =>
{
    console.log(`Server listening on http://localhost:${ port }!`);
});
